@@ -3,7 +3,8 @@ from __future__ import annotations
 import pytest
 
 from rapidfuzz import fuzz, process_cpp, process_py
-from rapidfuzz.distance import Levenshtein, Levenshtein_py
+from rapidfuzz.distance import DamerauLevenshtein, Levenshtein, Levenshtein_py
+from rapidfuzz.utils import default_process
 
 
 def wrapped(func):
@@ -52,6 +53,18 @@ class process:
             assert np.array_equal(res1, res2)
         return res1
 
+    @staticmethod
+    def cpdist(*args, **kwargs):
+        import numpy as np
+
+        res1 = process_cpp.cpdist(*args, **kwargs)
+        res2 = process_py.cpdist(*args, **kwargs)
+        assert res1.dtype == res2.dtype
+        assert res1.shape == res2.shape
+        if res1.size and res2.size:
+            assert np.array_equal(res1, res2)
+        return res1
+
 
 baseball_strings = [
     "new york mets vs chicago cubs",
@@ -192,6 +205,28 @@ def test_with_processor():
     best = process.extractOne(query, events, processor=lambda event: event[0])
     assert best[0] == events[0]
 
+    best = process.extract(query, events, processor=lambda event: event[0])[0]
+    assert best[0] == events[0]
+
+    eventsDict = dict(enumerate(events))
+    best = process.extractOne(query, eventsDict, processor=lambda event: event[0])
+    assert best[0] == events[0]
+
+    best = process.extract(query, eventsDict, processor=lambda event: event[0])[0]
+    assert best[0] == events[0]
+
+    best = process.extractOne("new york mets", ["new YORK mets"])
+    assert 72 < best[1] < 73
+
+    best = process.extract("new york mets", ["new YORK mets"])[0]
+    assert 72 < best[1] < 73
+
+    best = process.extractOne("new york mets", ["new YORK mets"], processor=default_process)
+    assert best[1] == 100
+
+    best = process.extract("new york mets", ["new YORK mets"], processor=default_process)[0]
+    assert best[1] == 100
+
 
 def test_with_scorer():
     choices = [
@@ -278,15 +313,65 @@ def test_none_elements():
     """
     when a None element is used, it is skipped and the index is still correct
     """
+    # no processor
     best = process.extractOne("test", [None, "tes"])
     assert best[2] == 1
     best = process.extractOne(None, [None, "tes"])
     assert best is None
+    best = process.extractOne("test", {0: None, 1: "tes"})
+    assert best[2] == 1
+    best = process.extractOne(None, {0: None, 1: "tes"})
+    assert best is None
 
+    # C++ processor
+    best = process.extractOne("test", [None, "tes"], processor=default_process)
+    assert best[2] == 1
+    best = process.extractOne(None, [None, "tes"], processor=default_process)
+    assert best is None
+    best = process.extractOne("test", {0: None, 1: "tes"}, processor=default_process)
+    assert best[2] == 1
+    best = process.extractOne(None, {0: None, 1: "tes"}, processor=default_process)
+    assert best is None
+
+    # python processor
+    best = process.extractOne("test", [None, "tes"], processor=lambda s: s)
+    assert best[2] == 1
+    best = process.extractOne(None, [None, "tes"], processor=lambda s: s)
+    assert best is None
+    best = process.extractOne("test", {0: None, 1: "tes"}, processor=lambda s: s)
+    assert best[2] == 1
+    best = process.extractOne(None, {0: None, 1: "tes"}, processor=lambda s: s)
+    assert best is None
+
+    # no processor
     best = process.extract("test", [None, "tes"])
     assert best[0][2] == 1
     best = process.extract(None, [None, "tes"])
     assert best == []
+    best = process.extract("test", {0: None, 1: "tes"})
+    assert best[0][2] == 1
+    best = process.extract(None, {0: None, 1: "tes"})
+    assert best == []
+
+    # C++ processor
+    best = process.extract("test", [None, "tes"], processor=default_process)
+    assert best[0][2] == 1
+    best = process.extract(None, [None, "tes"], processor=default_process)
+    assert best == []
+    best = process.extract("test", {0: None, 1: "tes"}, processor=default_process)
+    assert best[0][2] == 1
+    best = process.extract(None, {0: None, 1: "tes"}, processor=default_process)
+    assert best == []
+
+    # python processor
+    best = process.extract("test", [None, "tes"], processor=lambda s: s)
+    assert best[0][2] == 1
+    best = process.extract(None, [None, "tes"], processor=lambda s: s)
+    assert best == []
+    best = process.extract("test", {0: None, 1: "tes"}, processor=lambda s: s)
+    assert best[0][2] == 1
+    best = process.extract(None, {0: None, 1: "tes"}, processor=lambda s: s)
+    assert best == []
 
 
 def test_numpy_nan_elements():
@@ -294,15 +379,65 @@ def test_numpy_nan_elements():
     when a np.nan element is used, it is skipped and the index is still correct
     """
     np = pytest.importorskip("numpy")
+    # no processor
     best = process.extractOne("test", [np.nan, "tes"])
     assert best[2] == 1
     best = process.extractOne(np.nan, [np.nan, "tes"])
     assert best is None
+    best = process.extractOne("test", {0: np.nan, 1: "tes"})
+    assert best[2] == 1
+    best = process.extractOne(np.nan, {0: np.nan, 1: "tes"})
+    assert best is None
 
+    # C++ processor
+    best = process.extractOne("test", [np.nan, "tes"], processor=default_process)
+    assert best[2] == 1
+    best = process.extractOne(np.nan, [np.nan, "tes"], processor=default_process)
+    assert best is None
+    best = process.extractOne("test", {0: np.nan, 1: "tes"}, processor=default_process)
+    assert best[2] == 1
+    best = process.extractOne(np.nan, {0: np.nan, 1: "tes"}, processor=default_process)
+    assert best is None
+
+    # python processor
+    best = process.extractOne("test", [np.nan, "tes"], processor=lambda s: s)
+    assert best[2] == 1
+    best = process.extractOne(np.nan, [np.nan, "tes"], processor=lambda s: s)
+    assert best is None
+    best = process.extractOne("test", {0: np.nan, 1: "tes"}, processor=lambda s: s)
+    assert best[2] == 1
+    best = process.extractOne(np.nan, {0: np.nan, 1: "tes"}, processor=lambda s: s)
+    assert best is None
+
+    # no processor
     best = process.extract("test", [np.nan, "tes"])
     assert best[0][2] == 1
     best = process.extract(np.nan, [np.nan, "tes"])
     assert best == []
+    best = process.extract("test", {0: np.nan, 1: "tes"})
+    assert best[0][2] == 1
+    best = process.extract(np.nan, {0: np.nan, 1: "tes"})
+    assert best == []
+
+    # C++ processor
+    best = process.extract("test", [np.nan, "tes"], processor=default_process)
+    assert best[0][2] == 1
+    best = process.extract(np.nan, [np.nan, "tes"], processor=default_process)
+    assert best == []
+    best = process.extract("test", {0: np.nan, 1: "tes"}, processor=default_process)
+    assert best[0][2] == 1
+    best = process.extract(np.nan, {0: np.nan, 1: "tes"}, processor=default_process)
+    assert best == []
+
+    # python processor
+    best = process.extract("test", [np.nan, "tes"], processor=lambda s: s)
+    assert best[0][2] == 1
+    best = process.extract(np.nan, [np.nan, "tes"], processor=lambda s: s)
+    assert best == []
+    best = process.extract("test", {0: np.nan, 1: "tes"}, processor=lambda s: s)
+    assert best[0][2] == 1
+    best = process.extract(np.nan, {0: np.nan, 1: "tes"}, processor=lambda s: s)
+    assert best == []
 
 
 def test_pandas_nan_elements():
@@ -310,15 +445,65 @@ def test_pandas_nan_elements():
     when a pd.NA element is used, it is skipped and the index is still correct
     """
     pd = pytest.importorskip("pandas")
+    # no processor
     best = process.extractOne("test", [pd.NA, "tes"])
     assert best[2] == 1
     best = process.extractOne(pd.NA, [pd.NA, "tes"])
     assert best is None
+    best = process.extractOne("test", {0: pd.NA, 1: "tes"})
+    assert best[2] == 1
+    best = process.extractOne(pd.NA, {0: pd.NA, 1: "tes"})
+    assert best is None
 
+    # C++ processor
+    best = process.extractOne("test", [pd.NA, "tes"], processor=default_process)
+    assert best[2] == 1
+    best = process.extractOne(pd.NA, [pd.NA, "tes"], processor=default_process)
+    assert best is None
+    best = process.extractOne("test", {0: pd.NA, 1: "tes"}, processor=default_process)
+    assert best[2] == 1
+    best = process.extractOne(pd.NA, {0: pd.NA, 1: "tes"}, processor=default_process)
+    assert best is None
+
+    # python processor
+    best = process.extractOne("test", [pd.NA, "tes"], processor=lambda s: s)
+    assert best[2] == 1
+    best = process.extractOne(pd.NA, [pd.NA, "tes"], processor=lambda s: s)
+    assert best is None
+    best = process.extractOne("test", {0: pd.NA, 1: "tes"}, processor=lambda s: s)
+    assert best[2] == 1
+    best = process.extractOne(pd.NA, {0: pd.NA, 1: "tes"}, processor=lambda s: s)
+    assert best is None
+
+    # no processor
     best = process.extract("test", [pd.NA, "tes"])
     assert best[0][2] == 1
     best = process.extract(pd.NA, [pd.NA, "tes"])
     assert best == []
+    best = process.extract("test", {0: pd.NA, 1: "tes"})
+    assert best[0][2] == 1
+    best = process.extract(pd.NA, {0: pd.NA, 1: "tes"})
+    assert best == []
+
+    # C++ processor
+    best = process.extract("test", [pd.NA, "tes"], processor=default_process)
+    assert best[0][2] == 1
+    best = process.extract(pd.NA, [pd.NA, "tes"], processor=default_process)
+    assert best == []
+    best = process.extract("test", {0: pd.NA, 1: "tes"}, processor=default_process)
+    assert best[0][2] == 1
+    best = process.extract(pd.NA, {0: pd.NA, 1: "tes"}, processor=default_process)
+    assert best == []
+
+    # python processor
+    best = process.extract("test", [pd.NA, "tes"], processor=lambda s: s)
+    assert best[0][2] == 1
+    best = process.extract(pd.NA, [pd.NA, "tes"], processor=lambda s: s)
+    assert best == []
+    best = process.extract("test", {0: pd.NA, 1: "tes"}, processor=lambda s: s)
+    assert best[0][2] == 1
+    best = process.extract(pd.NA, {0: pd.NA, 1: "tes"}, processor=lambda s: s)
+    assert best == []
 
 
 def test_result_order():
@@ -391,6 +576,17 @@ def test_none_strings():
         assert scores[0, 3] == 0
         assert scores[0, 4] == 0
 
+        scores = process.cdist(["", None], ["", None], scorer=DamerauLevenshtein.normalized_similarity)
+        assert scores[0, 0] == 1
+        assert scores[0, 1] == 0
+        assert scores[1, 0] == 0
+        assert scores[1, 1] == 0
+
+        scores = process.cpdist(choices, choices)
+        assert scores[0] == 0
+        assert scores[3] == 0
+        assert scores[4] == 0
+
 
 def test_issue81():
     # this mostly tests whether this segfaults due to incorrect ref counting
@@ -437,6 +633,12 @@ def test_cdist_empty_seq(scorer):
     assert process.cdist(["a", "b"], [], scorer=scorer).shape == (2, 0)
 
 
+@pytest.mark.parametrize("scorer", [fuzz.ratio, fuzz.WRatio, custom_scorer])
+def test_cpdist_empty_seq(scorer):
+    pytest.importorskip("numpy")
+    assert process.cpdist([], [], scorer=scorer).shape == (0,)
+
+
 @pytest.mark.parametrize("scorer", [fuzz.ratio])
 def test_wrapped_function(scorer):
     pytest.importorskip("numpy")
@@ -444,6 +646,9 @@ def test_wrapped_function(scorer):
     assert process.cdist(["test"], [float("nan")], scorer=scorer)[0, 0] == 100
     assert process.cdist(["test"], [None], scorer=scorer)[0, 0] == 100
     assert process.cdist(["test"], ["tes"], scorer=scorer)[0, 0] == 100
+    assert process.cpdist(["test"], [float("nan")], scorer=scorer)[0] == 100
+    assert process.cpdist(["test"], [None], scorer=scorer)[0] == 100
+    assert process.cpdist(["test"], ["tes"], scorer=scorer)[0] == 100
 
     try:
         import pandas as pd
@@ -452,6 +657,7 @@ def test_wrapped_function(scorer):
 
     if pd is not None:
         assert process.cdist(["test"], [pd.NA], scorer=scorer)[0, 0] == 100
+        assert process.cpdist(["test"], [pd.NA], scorer=scorer)[0] == 100
 
 
 def test_cdist_not_symmetric():
@@ -464,7 +670,33 @@ def test_cdist_not_symmetric():
     )
 
 
-def test_cdist_muliplier():
+def test_cpdist_not_same_length():
+    pytest.importorskip("numpy")
+    with pytest.raises(ValueError, match="Length of queries and choices must be the same!"):
+        process.cpdist(["a", "b"], [])
+    with pytest.raises(ValueError, match="Length of queries and choices must be the same!"):
+        process.cpdist(["a", "b"], ["f"])
+
+
+def test_cpdist_multiplier():
+    np = pytest.importorskip("numpy")
+    string1 = ["test"]
+    string2 = ["test2"]
+    expected_res = np.array([204])
+    assert np.array_equal(
+        process.cpdist(
+            string1, string2, scorer=Levenshtein.normalized_similarity, score_multiplier=255, dtype=np.uint8
+        ),
+        expected_res,
+    )
+    expected_res = np.array([51])
+    assert np.array_equal(
+        process.cpdist(string1, string2, scorer=Levenshtein.normalized_distance, score_multiplier=255, dtype=np.uint8),
+        expected_res,
+    )
+
+
+def test_cdist_multiplier():
     np = pytest.importorskip("numpy")
     strings = ["test", "test2"]
     expected_res = np.array([[255, 204], [204, 255]])
@@ -514,3 +746,23 @@ def test_cdist_pure_python_dtype():
     assert process.cdist(["test"], ["test"], scorer=Levenshtein_py.similarity).dtype == np.uint32
     assert process.cdist(["test"], ["test"], scorer=Levenshtein_py.normalized_distance).dtype == np.float32
     assert process.cdist(["test"], ["test"], scorer=Levenshtein_py.normalized_similarity).dtype == np.float32
+
+
+def test_cpdist_pure_python_dtype():
+    np = pytest.importorskip("numpy")
+    assert process.cpdist(["test"], ["test"], scorer=Levenshtein_py.distance).dtype == np.uint32
+    assert process.cpdist(["test"], ["test"], scorer=Levenshtein_py.similarity).dtype == np.uint32
+    assert process.cpdist(["test"], ["test"], scorer=Levenshtein_py.normalized_distance).dtype == np.float32
+    assert process.cpdist(["test"], ["test"], scorer=Levenshtein_py.normalized_similarity).dtype == np.float32
+
+
+def test_cpdist_integral_dtype_rounding():
+    np = pytest.importorskip("numpy")
+    str1 = ["1 2 33 5"]
+    str2 = ["1 22 33 55"]
+    float_result = process.cpdist(str1, str2, dtype=np.float32)[0]
+    int_result = process.cpdist(str1, str2, dtype=np.uint32)[0]
+    # Check that the float result should be rounded up when in integer format
+    assert float_result % 1 >= 0.5
+    # Check if the rounded float result is equal to the integer result
+    assert round(float_result) == int_result
